@@ -43,10 +43,14 @@ class Configuration(object):
     values for fields not set.  You can also use the equivalent method to determine if two instances of this
     object have the same configuration content.
 
+    You may also use environment variable substitution in any string value in the configuration file.  You just
+    need to define the ``import_vars`` configuration field to be a list of variable names to import from the
+    shell and then use the $VAR_NAME in any string field.
+
     This also handles reporting status information about the configuration state, including what time it was
     read and what error (if any) was raised.
     """
-    def __init__(self, file_path, log_factory, monitor_factory):
+    def __init__(self, file_path, default_paths, default_monitors, log_factory, monitor_factory):
         self.__file_path = os.path.abspath(file_path)
         # Paths for additional configuration files that were read (from the config directory).
         self.__additional_paths = []
@@ -64,6 +68,19 @@ class Configuration(object):
         # each monitor entry that was found in the configuration file or implicit monitors.
         self.__monitors = []
 
+        # The DefaultPaths object that specifies the default paths for things like the data and log directory
+        # based on platform.
+        self.__default_paths = default_paths
+
+        # A list of dict/JsonObjects specifying the monitors to run by default.  This comes from the PlatformController
+        # to automatically include platform-specific monitors.  On Linux machines, this would include the
+        # linux_system_metrics and linux_process_metrics for the agent process.
+        self.__default_monitors = default_monitors
+
+        # Used to implement variable substitution in configuration file values.  This maps the variable name
+        # to the value to use in its place.
+        self.__substitutions = {}
+
         # FIX THESE:
         # Add documentation, verify, etc.
         self.__log_factory = log_factory
@@ -84,6 +101,10 @@ class Configuration(object):
             except JsonReadFileException, e:
                 raise BadConfiguration(str(e), None, 'fileParseError')
 
+            # Import any requested variables from the shell and use them for substitutions.
+            self.__import_shell_variables()
+            self.__perform_substitutions()
+
             self.__verify_main_config_and_apply_defaults(self.__config, self.__file_path)
             self.__verify_logs_and_monitors_configs_and_apply_defaults(self.__config, self.__file_path)
 
@@ -117,20 +138,12 @@ class Configuration(object):
                 self.__verify_log_entry_and_set_defaults(config, description='implicit rule')
                 agent_log = config
 
-            # Add in implicit entry to run the metric monitor collector.
-            metric_monitor = None
-            if self.implicit_metric_monitor:
-                config = JsonObject(module='scalyr_agent.builtin_monitors.linux_system_metrics')
-                self.__verify_monitor_entry_and_set_defaults(config, "implicit rules", -1)
-                metric_monitor = config
-
-            # Add in an implicit instance of the apps_metric monitor for the agent.
-            agent_apps_metric_monitor = None
-            if self.implicit_agent_process_metrics_monitor:
-                config = JsonObject(module='scalyr_agent.builtin_monitors.linux_process_metrics',
-                                    pid='$$', id='agent')
-                self.__verify_monitor_entry_and_set_defaults(config, "implicit rules", -1)
-                agent_apps_metric_monitor = config
+            # Add in any platform-specific monitors.
+            platform_monitors = []
+            for monitor in self.__default_monitors:
+                config = JsonObject(content=monitor)
+                self.__verify_monitor_entry_and_set_defaults(config, 'default monitors for platform', -1)
+                platform_monitors.append(config)
 
             all_logs = list(self.__config.get_json_array('logs'))
             if agent_log is not None:
@@ -144,10 +157,8 @@ class Configuration(object):
             # Tracks which modules already had an id present in the module config.
             had_id = {}
             all_monitors = list(self.__config.get_json_array('monitors'))
-            if metric_monitor is not None:
-                all_monitors.append(metric_monitor)
-            if agent_apps_metric_monitor is not None:
-                all_monitors.append(agent_apps_metric_monitor)
+            for monitor in platform_monitors:
+                all_monitors.append(monitor)
 
             for entry in all_monitors:
                 module_name = entry['module'].split('.')[-1]
@@ -413,62 +424,12 @@ class Configuration(object):
             if original_debug_level is not None:
                 other.__config.put('debug_level', original_debug_level)
 
-    @staticmethod
-    def is_tarball_install():
-        """Returns True if this agent was installed using the tarball method.
-        """
-        return Configuration.__determine_platform_type() == 'tarball'
-
-    @staticmethod
-    def default_agent_log_path():
-        """Returns the default log path for the agent."""
-        # TODO:  Support more platforms.
-        if Configuration.__determine_platform_type() == 'tarball':
-            return Configuration.__resolve_to_install_location('log')
-        else:
-            return '/var/log/scalyr-agent-2'
-
-    @staticmethod
-    def default_agent_data_path():
-        # TODO:  Support more platforms.
-        if Configuration.__determine_platform_type() == 'tarball':
-            return Configuration.__resolve_to_install_location('data')
-        else:
-            return '/var/lib/scalyr-agent-2'
-
-    @staticmethod
-    def default_config_file_path():
-        """Returns the default configuration file path for the agent."""
-        # TODO:  Support more platforms.
-        if Configuration.__determine_platform_type() == 'tarball':
-            return Configuration.__resolve_to_install_location('config', 'agent.json')
-        else:
-            return '/etc/scalyr-agent-2/agent.json'
-
     @staticmethod
     def default_ca_cert_path():
         """Returns the default configuration file path for the agent."""
         # TODO:  Support more platforms.
         return Configuration.__resolve_to_install_location('certs', 'ca_certs.crt')
 
-    @staticmethod
-    def __determine_platform_type():
-        """Returns a string identifying the installation method that was used to install the agent, as well
-        as the platform.
-
-        TODO:  This method doesn't return all platforms yet, just 'tarball' or 'linux'.
-        """
-        # See if the source was installed using the tarball method, which
-        # is identified by 'packageless' being a file in root of the install location for the source.
-        # (Typically, the path looks like /usr/share/scalyr-agent-2/py/scalyr_agent/configuration.py,
-        # where scalyr-agent-2 is the install location).
-        tarball_install = os.path.exists(os.path.join(get_install_root(), 'packageless'))
-
-        if tarball_install:
-            return 'tarball'
-        else:
-            return 'linux'
-
     @staticmethod
     def __resolve_to_install_location(*paths):
         """Returns the absolute path created by joining the specified intermediate paths to
@@ -565,9 +526,9 @@ class Configuration(object):
                                    file_path, 'api_key', 'emptyApiKey')
 
         self.__verify_or_set_optional_attributes(config, 'server_attributes', description)
-        self.__verify_or_set_optional_string(config, 'agent_log_path', Configuration.default_agent_log_path(),
+        self.__verify_or_set_optional_string(config, 'agent_log_path', self.__default_paths.agent_log_path,
                                              description)
-        self.__verify_or_set_optional_string(config, 'agent_data_path', Configuration.default_agent_data_path(),
+        self.__verify_or_set_optional_string(config, 'agent_data_path',  self.__default_paths.agent_data_path,
                                              description)
         self.__verify_or_set_optional_string(config, 'additional_monitor_module_paths', '', description)
         self.__verify_or_set_optional_string(config, 'scalyr_server', 'https://agent.scalyr.com', description)
@@ -944,6 +905,86 @@ class Configuration(object):
             raise BadConfiguration(self.__last_error, 'fake', 'fake')
         return self.__config
 
+    def __import_shell_variables(self):
+        """Imports the shell variables requested in 'import_vars' and adds them to self.__substitutions.
+        """
+        if 'import_vars' in self.__config:
+            for var_name in self.__config.get_json_array('import_vars'):
+                if var_name in os.environ:
+                    self.__substitutions[var_name] = os.environ[var_name]
+                else:
+                    self.__substitutions[var_name] = ''
+
+    def __perform_substitutions(self):
+        """Rewrites self.__config to reflect any substitutions in self.__substitutions."""
+
+        def perform_generic_substitution(value):
+            """Takes a given JSON value and performs the appropriate substitution.
+
+            This method will return a non-None value if the value has to be replaced with the returned value.
+            Otherwise, this will attempt to perform in-place substitutions.
+
+            For str, unicode, it substitutes the variables and returns the result.  For
+            container objects, it does the recursive substitution.
+
+            @param value: The JSON value
+            @type value: Any valid element of a JsonObject
+            @return: The value that should replace the original, if any.  If no replacement is necessary, returns None
+            """
+            result = None
+            value_type = type(value)
+
+            if (value_type is str or value_type is unicode) and '$' in value:
+                result = perform_str_substitution(value)
+            elif isinstance(value, JsonObject):
+                perform_object_substitution(value)
+            elif isinstance(value, JsonArray):
+                perform_array_substitution(value)
+            return result
+
+        def perform_object_substitution(object_value):
+            """Performs the in-place substitution for a JsonObject.
+
+            @param object_value: The object to perform substitutions on.
+            @type object_value: JsonObject
+            """
+            # We collect the new values and apply them later to avoid messing up the iteration.
+            new_values = {}
+            for (key, value) in object_value.iteritems():
+                replace_value = perform_generic_substitution(value)
+                if replace_value is not None:
+                    new_values[key] = replace_value
+
+            for (key, value) in new_values.iteritems():
+                object_value[key] = value
+
+        def perform_str_substitution(str_value):
+            """Performs substitutions on the given string.
+
+            @param str_value: The input string.
+            @type str_value: str or unicode
+            @return: The resulting value after substitution.
+            @rtype: str or unicode
+            """
+            result = str_value
+            for (var_name, value) in self.__substitutions.iteritems():
+                result = result.replace('$%s' % var_name, value)
+            return result
+
+        def perform_array_substitution(array_value):
+            """Perform substitutions on the JsonArray.
+
+            @param array_value: The array
+            @type array_value: JsonArray
+            """
+            for i in range(len(array_value)):
+                replace_value = perform_generic_substitution(array_value[i])
+                if replace_value is not None:
+                    array_value[i] = replace_value
+
+        # Actually do the work.
+        perform_object_substitution(self.__config)
+
 
 class BadConfiguration(Exception):
     """Raised when bad values are supplied in the configuration."""
