@@ -14,6 +14,7 @@
 # ------------------------------------------------------------------------
 #
 # author: Steven Czerwinski <czerwin@scalyr.com>
+from scalyr_agent.platform_controller import DefaultPaths
 
 __author__ = 'czerwin@scalyr.com'
 
@@ -415,9 +416,7 @@ class TestConfiguration(unittest.TestCase):
 
     def test_parser_specification(self):
         self.__write_file(""" {
-            implicit_metric_monitor: false,
             implicit_agent_log_collection: false,
-            implicit_agent_process_metrics_monitor: false,
             api_key: "hi there",
             logs: [ { path: "/tmp/foo.txt",
                       parser: "foo-parser"} ]
@@ -425,7 +424,7 @@ class TestConfiguration(unittest.TestCase):
         """)
         config = self.__create_test_configuration_instance()
         config.parse()
-        self.assertEquals(len(config.logs), 1)
+        self.assertEquals(len(config.logs), 3)
         self.assertEquals(config.logs[0].config['attributes']['parser'], 'foo-parser')
 
     def test_monitors(self):
@@ -558,6 +557,48 @@ class TestConfiguration(unittest.TestCase):
 
         self.assertTrue(error_seen)
 
+    def test_substitution(self):
+        self.__write_file(""" {
+            import_vars: [ "TEST_VAR" ],
+            api_key: "hi$TEST_VAR",
+          }
+        """)
+
+        os.environ['TEST_VAR'] = 'bye'
+        config = self.__create_test_configuration_instance()
+        config.parse()
+
+        self.assertEquals(config.api_key, 'hibye')
+
+    def test_json_array_substitution(self):
+        self.__write_file(""" {
+            import_vars: [ "TEST_VAR", "DIR_VAR" ],
+            api_key: "hi$TEST_VAR",
+            logs: [ { path:"/var/log/tomcat6/$DIR_VAR.log" }]
+          }
+        """)
+
+        os.environ['TEST_VAR'] = 'bye'
+        os.environ['DIR_VAR'] = 'ok'
+
+        config = self.__create_test_configuration_instance()
+        config.parse()
+
+        self.assertEquals(config.api_key, 'hibye')
+        self.assertEquals(config.logs[0].config.get_string('path'), '/var/log/tomcat6/ok.log')
+
+    def test_empty_substitution(self):
+        self.__write_file(""" {
+            import_vars: [ "UNDEFINED_VAR" ],
+            api_key: "hi$UNDEFINED_VAR",
+          }
+        """)
+
+        config = self.__create_test_configuration_instance()
+        config.parse()
+
+        self.assertEquals(config.api_key, 'hi')
+
     def __write_file(self, contents):
         fp = open(self.__config_file, 'w')
         fp.write(contents)
@@ -588,4 +629,10 @@ class TestConfiguration(unittest.TestCase):
         def monitor_factory(config, _):
             return TestConfiguration.MonitorObject(config)
 
-        return Configuration(self.__config_file, log_factory, monitor_factory)
\ No newline at end of file
+        default_paths = DefaultPaths('/var/log/scalyr-agent-2', '/etc/scalyr-agent-2/agent.json',
+                                     '/var/lib/scalyr-agent-2')
+
+        monitors = [JsonObject(module='scalyr_agent.builtin_monitors.linux_system_metrics'),
+                    JsonObject(module='scalyr_agent.builtin_monitors.linux_process_metrics',
+                               pid='$$', id='agent')]
+        return Configuration(self.__config_file, default_paths, monitors, log_factory, monitor_factory)
\ No newline at end of file
